#define BP_DEBUG	//comment out to prevent debug messages and asserts
	
#ifdef BP_DEBUG
    #define DebugLog(x)			Sys.Print("" + x)
    #define DebugError(x)		Sys.Warning("" + x + "\n")
	#define DebugLogThis(x)		DebugLog(#x + " = " + x)
	#define DebugLogIsNull(x)	DebugLog(#x + " is null = " + (x is null))
    #define DebugLogAssert(x)	DebugError("[ASSERT] " + x)
	namespace BP
	{
		void Debug_Assertion_Failed_Check_stdout_File_For_Details_Right_Now(bool condition, const kStr& in message)
		{
			if (!condition)
			{
				DebugLogAssert(message);
				array<int> doCrash = { doCrash[0] }; //Crash game and exit with decent runtime error message
			}
		}
	}
	#define DebugAssert(condition, message)	BP::Debug_Assertion_Failed_Check_stdout_File_For_Details_Right_Now(condition, message)
#else
    #define DebugLog(x)
    #define DebugError(x)
	#define DebugLogThis(x)
	#define DebugLogIsNull(x)
	#define DebugLogAssert(x)
	#define DebugAssert(x, y)
#endif

#define PrintLog(x)				Sys.Print("" + x)
#define PrintError(x)			Sys.Warning("" + x + "\n")

#define SERIALIZE_INT(x)		dict.Add(#x, ""+x)
#define SERIALIZE_FLOAT(x)		dict.Add(#x, ""+x)
#define SERIALIZE_BOOL(x)		dict.Add(#x, ""+x)
#define SERIALIZE_STRING(x)		dict.Add(#x, x)
#define SERIALIZE_VECTOR(x)		dict.Add(#x, x.ToString())
#define DESERIALIZE_STRING(x)	dict.GetString(#x, x)
#define DESERIALIZE_VECTOR(x)	dict.GetVector(#x, x)

#define SERIALIZEKEY_INT(key, x)		dict.Add(key + #x, ""+x)
#define SERIALIZEKEY_FLOAT(key, x)		dict.Add(key + #x, ""+x)
#define SERIALIZEKEY_BOOL(key, x)		dict.Add(key + #x, ""+x)
#define SERIALIZEKEY_STRING(key, x)		dict.Add(key + #x, x)
#define SERIALIZEKEY_VECTOR(key, x)		dict.Add(key + #x, x.ToString())
#define DESERIALIZEKEY_FLOAT(key, x)	dict.GetFloat(key + #x, x)
#define DESERIALIZEKEY_INT(key, x)		dict.GetInt(key + #x, x)
#define DESERIALIZEKEY_BOOL(key, x)		x = DeserializeBool(dict, key + #x)
#define DESERIALIZEKEY_STRING(key, x)	dict.GetString(key + #x, x)
#define DESERIALIZEKEY_VECTOR(key, x)	dict.GetVector(key + #x, x)

#define QUAT_IDENTITY kQuat(0.0f, 0.0f, 0.0f)
#define VEC_ZERO Math::vecZero

#define FOREACH_ACTOR \
	kActorIterator iter; \
	kActor@ actor; \
	while ((@actor = iter.GetNext()) !is null)
#define FOREACH2_ACTOR \
	kActorIterator iter2; \
	while ((@actor = iter2.GetNext()) !is null)
		
#define IsNetServer (Network.IsNetgame() && Network.IsServer())
#define IsNetClient (Network.IsNetgame() && Network.IsClient())

#define USEREVENT_MSG_SPAWN					1
#define USEREVENT_MSG_GET_SCRIPT			2
#define USEREVENT_MSG_AI_MOVE_POINT			3
#define USEREVENT_MSG_AI_ATTACK_POINT		4
#define USEREVENT_MSG_AI_STARTED			5
#define USEREVENT_MSG_AI_KILLED				6
#define USEREVENT_MSG_PLAYER_REVIVED        7
#define USEREVENT_MSG_PLAYER_STARTED        8
#define USEREVENT_MSG_MAP_NETRECEIVE        9
#define USEREVENT_MSG_MP_PLAYER_JOIN		10

#define MP_GAMEMODE_ROK				1
#define MP_GAMEMODE_TEAMROK			2
#define MP_GAMEMODE_FRAGTAG			3
#define MP_GAMEMODE_LASTSTANDING	4
#define MP_GAMEMODE_ROKMIX			5
#define MP_GAMEMODE_INSTAROK		6
#define MP_GAMEMODE_RAPTORFEST		7

#include "defs/BP/common.cpp"

#include "scripts/BP/Utils/Math.cpp"
#include "scripts/BP/Utils/String.cpp"
#include "scripts/BP/Utils/Array.cpp"
#include "scripts/BP/Utils/Actor.cpp"
#include "scripts/BP/Utils/Input.cpp"
#include "scripts/BP/Utils/Spawn.cpp"
#include "scripts/BP/Utils/UserEvent.cpp"
#include "scripts/BP/Utils/Inventory.cpp"
#include "scripts/BP/Utils/Cvars.cpp"

#include "scripts/BP/Actors/ScriptActor.cpp"
#include "scripts/BP/Actors/Player.cpp"

#include "scripts/BP/Actors/AI/AI.cpp"
#include "scripts/BP/Actors/AI/AraissiSoldier.cpp"
#include "scripts/BP/Actors/AI/BioBot.cpp"
#include "scripts/BP/Actors/AI/BlindGuardian.cpp"
#include "scripts/BP/Actors/AI/BlindSentinel.cpp"
#include "scripts/BP/Actors/AI/CaveSpider.cpp"
#include "scripts/BP/Actors/AI/CaveWorm.cpp"
#include "scripts/BP/Actors/AI/Compsognathus.cpp"
#include "scripts/BP/Actors/AI/Deadman.cpp"
#include "scripts/BP/Actors/AI/DeadmanHalf.cpp"
#include "scripts/BP/Actors/AI/DeathGuard.cpp"
#include "scripts/BP/Actors/AI/Endtrail.cpp"
#include "scripts/BP/Actors/AI/Fireborn.cpp"
#include "scripts/BP/Actors/AI/FleshSentinel.cpp"
#include "scripts/BP/Actors/AI/FleshWorm.cpp"
#include "scripts/BP/Actors/AI/Grub.cpp"
#include "scripts/BP/Actors/AI/HiveDrone.cpp"
#include "scripts/BP/Actors/AI/HiveSoldier.cpp"
#include "scripts/BP/Actors/AI/HiveWorker.cpp"
#include "scripts/BP/Actors/AI/Leaper.cpp"
#include "scripts/BP/Actors/AI/LordoftheDead.cpp"
#include "scripts/BP/Actors/AI/LordoftheFlesh.cpp"
#include "scripts/BP/Actors/AI/Mite.cpp"
#include "scripts/BP/Actors/AI/MotherGrub.cpp"
#include "scripts/BP/Actors/AI/Nala.cpp"
#include "scripts/BP/Actors/AI/PrimagenDrone.cpp"
#include "scripts/BP/Actors/AI/PrimagenGuard.cpp"
#include "scripts/BP/Actors/AI/PrimagenTrooper.cpp"
#include "scripts/BP/Actors/AI/PurlinGunner.cpp"
#include "scripts/BP/Actors/AI/PurlinJuggernaut.cpp"
#include "scripts/BP/Actors/AI/PurlinWarClub.cpp"
#include "scripts/BP/Actors/AI/Raptoid.cpp"
#include "scripts/BP/Actors/AI/Raptor.cpp"
#include "scripts/BP/Actors/AI/SisterofDespair.cpp"
#include "scripts/BP/Actors/AI/SpiderHatchling.cpp"
#include "scripts/BP/Actors/AI/SwampWasp.cpp"

#include "scripts/BP/Actors/Maps/ScriptMap.cpp"

#include "scripts/BP/Actors/Controllers/Flare.cpp"
#include "scripts/BP/Actors/Controllers/RazorWind.cpp"
#include "scripts/BP/Actors/Controllers/QuadMissileLauncher.cpp"
#include "scripts/BP/Actors/Controllers/CerebralBore.cpp"
#include "scripts/BP/Actors/Controllers/WorldChar.cpp"

#include "scripts/BP/Weapons/ScriptWeapon.cpp"
#include "scripts/BP/Weapons/Bow.cpp"
#include "scripts/BP/Weapons/CerebralBore.cpp"
#include "scripts/BP/Weapons/ChargeDart.cpp"
#include "scripts/BP/Weapons/FireStormCannon.cpp"
#include "scripts/BP/Weapons/FlameThrower.cpp"
#include "scripts/BP/Weapons/Flare.cpp"
#include "scripts/BP/Weapons/GrenadeLauncher.cpp"
#include "scripts/BP/Weapons/Harpoon.cpp"
#include "scripts/BP/Weapons/Mag60.cpp"
#include "scripts/BP/Weapons/Nuke.cpp"
#include "scripts/BP/Weapons/PFMLayer.cpp"
#include "scripts/BP/Weapons/Pistol.cpp"
#include "scripts/BP/Weapons/PlasmaRifle.cpp"
#include "scripts/BP/Weapons/QuadMissileLauncher.cpp"
#include "scripts/BP/Weapons/RaptorClaws.cpp"
#include "scripts/BP/Weapons/RazorWind.cpp"
#include "scripts/BP/Weapons/RidingGun.cpp"
#include "scripts/BP/Weapons/Shotgun.cpp"
#include "scripts/BP/Weapons/Shredder.cpp"
#include "scripts/BP/Weapons/SunfirePod.cpp"
#include "scripts/BP/Weapons/Talons.cpp"
#include "scripts/BP/Weapons/TekBow.cpp"
#include "scripts/BP/Weapons/TorpedoLauncher.cpp"
#include "scripts/BP/Weapons/Tranquilizer.cpp"
#include "scripts/BP/Weapons/WarBlade.cpp"
#include "scripts/BP/Weapons/AssaultRifleMP.cpp"
#include "scripts/BP/Weapons/CerebralBoreMP.cpp"
#include "scripts/BP/Weapons/ChargeDartMP.cpp"
#include "scripts/BP/Weapons/CrossbowMP.cpp"
#include "scripts/BP/Weapons/FireStormCannonMP.cpp"
#include "scripts/BP/Weapons/GrenadeLauncherMP.cpp"
#include "scripts/BP/Weapons/PlasmaRifleMP.cpp"
#include "scripts/BP/Weapons/QuadMissileLauncherMP.cpp"

#include "scripts/BP/Server/ConsoleCommands.cpp"
#include "scripts/BP/Server/NetworkCommands.cpp"

