#define NETMSG_TIME 300
#define NETMSG_GRAVITY "Player Gravity set to " + BP::playerGravity
#define NETMSG_SPEED "Player Speed set to " + BP::playerSpeed
#define NETMSG_DAMAGE "Player Damage set to " + BP::playerDamage
#define NETMSG_RECEIVEDAMAGE "Player Receive Damage set to " + BP::playerReceiveDamage
#define NETMSG_DOUBLE_JUMP "Double Jump " + (BP::playerCanDoubleJump ? "Enabled" : "Disabled")
#define NETMSG_GROUND_DASH "Ground Dash " + (BP::playerCanGroundDash ? "Enabled" : "Disabled")
#define NETMSG_AIR_DASH "Air Dash " + (BP::playerCanAirDash ? "Enabled" : "Disabled")
#define NETMSG_GROUND_POUND "Ground Pound " + (BP::playerCanGroundPound ? "Enabled" : "Disabled")
#define NETMSG_SHOW_PLAYER_DAMAGE "Show Player Damage " + (BP::showPlayerDamage ? "Enabled" : "Disabled")
#define NETMSG_CLOAK "Player Cloaking " + (BP::playerCloaking ? "Enabled" : "Disabled")

void SendSound(const int soundID)
{
	NetWorkWrite32("void NetworkReceiveSound(void)", soundID);
}

void SetMapNetData(const int mapData, const int mapData2)
{
	NetWorkWrite32("void NetworkReceiveMapNetData(void)", mapData, mapData2);
}

void PlaySoundOnNonHiddenActor(const int soundID)
{
	FOREACH_ACTOR
	{
		if ((actor.Flags() & AF_HIDDEN) == 0)
		{
			actor.PlaySound(soundID);
			return;
		}
	}
}

void slap(const int16 actorNetID)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		kActor@ actor = Network.GetNetActor(actorNetID);
		if (@actor == null || BP::Actor::IsDead(@actor))
		{
			PrintLog("Can not slap player " + actorNetID);
			return;
		}
		int x = Math::RandMax(16);
		int y = Math::RandMax(16);
		int z = Math::RandMax(16);
		int power = Math::RandMax(16);
		int16 flags = (x << 0) | (y << 4) | (z << 8) | (power << 12);
		SlapPlayer(actorNetID, flags);
		PrintLog("Slapped player " + actorNetID);
		NetWorkWrite("void NetworkReceivePlayerSlap(void)", actorNetID, flags);
	}
}

void SlapPlayer(int16 actorNetID, int16 flags)
{
	kActor@ actor = Network.GetNetActor(actorNetID);
	if (@actor == null)
	{
		return;
	}
	
	if (actor.WorldComponent() !is null)
	{
		//not in water and has a movement component
		if ((actor.WorldComponent().Flags() & (WCF_UNDER_WATER | WCF_ON_WATER)) == 0 && actor.MovementComponent() !is null)
		{
			actor.MovementComponent().Flags() |= MCF_DISABLE_AIR_FRICTION_UNTIL_LAND;
		}
	}
	int16 x = ((flags & 0xf) >> 0) - 7;
	int16 y = ((flags & 0xf0) >> 4) - 7;
	int16 z = ((flags & 0xf00) >> 8) - 7;
	int16 power = 30 + ((flags & 0xf000) >> 12);	
	kVec3 vel = kVec3(x, y, float(z) / 2).Normalize();
	vel.x *= power;
	vel.y *= power;
	vel.z *= power * 0.8f;
	BP::slapVelocity = vel;
	actor.PlaySound(BP_SFX_Slap);
	if (actor.MovementComponent() !is null)
	{
		actor.MovementComponent().Velocity() = BP::slapVelocity;
	}
}

void showmods()
{
	PrintLog("----------Current Mods----------");
	PrintError(NETMSG_GRAVITY);
	PrintError(NETMSG_SPEED);
	PrintError(NETMSG_DAMAGE);
	PrintError(NETMSG_RECEIVEDAMAGE);
	PrintError(NETMSG_DOUBLE_JUMP);
	PrintError(NETMSG_GROUND_DASH);
	PrintError(NETMSG_AIR_DASH);
	PrintError(NETMSG_GROUND_POUND);
	PrintError(NETMSG_SHOW_PLAYER_DAMAGE);
	PrintError(NETMSG_CLOAK);
	PrintLog("");
}

void SetAllNetModsSilent()
{
	NetWorkWrite("void NetworkReceiveDoubleJumpChange(void)", BP::playerCanDoubleJump ? 1 : 0, 1);
	NetWorkWrite("void NetworkReceiveGroundDashChange(void)", BP::playerCanGroundDash ? 1 : 0, 1);
	NetWorkWrite("void NetworkReceiveAirDashChange(void)", BP::playerCanAirDash ? 1 : 0, 1);
	NetWorkWrite("void NetworkReceiveGroundPoundChange(void)", BP::playerCanGroundPound ? 1 : 0, 1);
	NetWorkWrite("void NetworkReceiveDamageChange(void)", int16(BP::playerReceiveDamage * 10.0f), 1);
	NetWorkWrite("void NetworkReceiveDefenseChange(void)", int16(BP::playerDamage * 10.0f), 1);
	NetWorkWrite("void NetworkReceiveSpeedChange(void)", int16(BP::playerSpeed * 100.0f), 1);
	NetWorkWrite("void NetworkReceiveGravityChange(void)", int16(BP::playerGravity * 100.0f), 1);
	NetWorkWrite("void NetworkReceiveShowPlayerDamageChange(void)", BP::showPlayerDamage ? 1 : 0, 1);
	NetWorkWrite("void NetworkReceivePlayerCloakChange(void)", BP::playerCloaking ? 1 : 0, 1);
	SendMPGameMode(BP::mpGameMode);
}

void SendMPGameMode(int gameMode)
{
	NetWorkWrite("void NetworkReceiveGameModeChange(void)", gameMode);
}

void SetPlayerCloak(const bool cloak)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerCloaking = cloak;
		SetCloakForAllActors(cloak);
		AddHudMessage(NETMSG_CLOAK);
		NetWorkWrite("void NetworkReceivePlayerCloakChange(void)", cloak ? 1 : 0, 0);
	}
}

void SetCloakForAllActors(const bool cloak)
{
	FOREACH_ACTOR
	{
		if (actor.Type() == kActor_Player)
		{
			if (cloak)
			{
				actor.RenderMeshComponent().Flags() |= RMCF_CLOAKED;
			}
			else
			{
				actor.RenderMeshComponent().Flags() &= ~RMCF_CLOAKED;
			}
		}
	}
}

void SetShowPlayerDamage(const bool showDamage)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::showPlayerDamage = showDamage;
		AddHudMessage(NETMSG_SHOW_PLAYER_DAMAGE);
		NetWorkWrite("void NetworkReceiveShowPlayerDamageChange(void)", showDamage ? 1 : 0, 0);
	}
}

void SetDoubleJump(const bool canDoubleJump)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerCanDoubleJump = canDoubleJump;
		AddHudMessage(NETMSG_DOUBLE_JUMP);
		NetWorkWrite("void NetworkReceiveDoubleJumpChange(void)", canDoubleJump ? 1 : 0, 0);
	}
}

void SetGroundDash(const bool canGroundDash)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerCanGroundDash = canGroundDash;
		AddHudMessage(NETMSG_GROUND_DASH);
		NetWorkWrite("void NetworkReceiveGroundDashChange(void)", canGroundDash ? 1 : 0, 0);
	}
}

void SetAirDash(const bool canAirDash)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerCanAirDash = canAirDash;
		AddHudMessage(NETMSG_AIR_DASH);
		NetWorkWrite("void NetworkReceiveAirDashChange(void)", canAirDash ? 1 : 0, 0);
	}
}

void SetGroundPound(const bool canGroundPound)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerCanGroundPound = canGroundPound;
		AddHudMessage(NETMSG_GROUND_POUND);
		NetWorkWrite("void NetworkReceiveGroundPoundChange(void)", canGroundPound ? 1 : 0, 0);
	}
}

void SetDamage(const float damage)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerDamage = damage;
		AddHudMessage(NETMSG_DAMAGE);
		NetWorkWrite("void NetworkReceiveDamageChange(void)", int16(damage * 10.0f), 0);
	}
}

void SetReceiveDamage(const float damage)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerReceiveDamage = damage;
		AddHudMessage(NETMSG_RECEIVEDAMAGE);
		NetWorkWrite("void NetworkReceiveDefenseChange(void)", int16(damage * 10.0f), 0);
	}
}

void SetSpeed(const float speed)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerSpeed = speed;
		SetSpeedForAllActors(speed);
		AddHudMessage(NETMSG_SPEED);
		NetWorkWrite("void NetworkReceiveSpeedChange(void)", int16(speed * 100.0f), 0);
	}
}

void SetSpeedForAllActors(const float speed)
{
	FOREACH_ACTOR
	{
		if (actor.Type() == kActor_Player)
		{
			BP::Actor::SetSpeed(@actor, speed);
		}
	}
}

void SetGravity(const float gravity)
{
	if (Network.IsNetgame() && Network.IsServer())
	{
		BP::playerGravity = gravity;
		SetGravityForAllActors(gravity);
		AddHudMessage(NETMSG_GRAVITY);
		NetWorkWrite("void NetworkReceiveGravityChange(void)", int16(gravity * 100.0f), 0);
	}
}

void SetGravityForAllActors(const float gravity)
{
	FOREACH_ACTOR
	{
		if (actor.Type() == kActor_Player)
		{
			BP::Actor::SetGravity(@actor, gravity);
		}
	}
}

void NetWorkWrite(const kStr&in header)
{
	NetSend::Clear();
	NetSend::WriteScriptHeader(header);
	Network.TransmitToAll();
}

void NetWorkWrite(const kStr&in header, const int16 data)
{
	NetSend::Clear();
	NetSend::WriteScriptHeader(header);
	NetSend::WriteInt16(data);
	Network.TransmitToAll();
}

void NetWorkWrite(const kStr&in header, const int16 data, const int16 data2)
{
	NetSend::Clear();
	NetSend::WriteScriptHeader(header);
	NetSend::WriteInt16(data);
	NetSend::WriteInt16(data2);
	Network.TransmitToAll();
}

void NetWorkWrite32(const kStr&in header, const int data)
{
	NetSend::Clear();
	NetSend::WriteScriptHeader(header);
	NetSend::WriteInt32(data);
	Network.TransmitToAll();
}

void NetWorkWrite32(const kStr&in header, const int data, const int data2)
{
	NetSend::Clear();
	NetSend::WriteScriptHeader(header);
	NetSend::WriteInt32(data);
	NetSend::WriteInt32(data2);
	Network.TransmitToAll();
}

void AddHudMessage(const kStr&in msg, const int flags = 0)
{
	bool silent = (flags & (1 << 0)) != 0;
	if (!silent)
	{
		Hud.AddMessage(msg, NETMSG_TIME);
	}
}

//-----Network callback functions-----
void NetworkReceiveSpeedChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerSpeed = (NetRecv::ReadInt16() / 100.0f); 
		SetSpeedForAllActors(BP::playerSpeed);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_SPEED, flags);
	}
}

void NetworkReceiveGravityChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerGravity = (NetRecv::ReadInt16() / 100.0f); 
		SetGravityForAllActors(BP::playerGravity);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_GRAVITY, flags);
	}
}

void NetworkReceiveDamageChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerDamage = (NetRecv::ReadInt16() / 10.0f); 
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_DAMAGE, flags);
	}
}

void NetworkReceiveDefenseChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerReceiveDamage = (NetRecv::ReadInt16() / 10.0f); 
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_RECEIVEDAMAGE, flags);
	}
}

void NetworkReceiveDoubleJumpChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerCanDoubleJump = (NetRecv::ReadInt16() == 1);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_DOUBLE_JUMP, flags);
	}
}

void NetworkReceiveGroundDashChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerCanGroundDash = (NetRecv::ReadInt16() == 1);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_GROUND_DASH, flags);
	}
}

void NetworkReceiveAirDashChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerCanAirDash = (NetRecv::ReadInt16() == 1);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_AIR_DASH, flags);
	}
}

void NetworkReceiveGroundPoundChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerCanGroundPound = (NetRecv::ReadInt16() == 1);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_GROUND_POUND, flags);
	}
}

void NetworkReceiveShowPlayerDamageChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::showPlayerDamage = (NetRecv::ReadInt16() == 1);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_SHOW_PLAYER_DAMAGE, flags);
	}
}

void NetworkReceivePlayerCloakChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::playerCloaking = (NetRecv::ReadInt16() == 1);
		SetCloakForAllActors(BP::playerCloaking);
		int16 flags = NetRecv::ReadInt16();
		AddHudMessage(NETMSG_CLOAK, flags);
	}
}

void NetworkReceivePlayerSlap(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		int16 actorNetID = NetRecv::ReadInt16();
		int16 flags = NetRecv::ReadInt16();
		SlapPlayer(actorNetID, flags);
	}
}

void NetworkReceiveGameModeChange(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		BP::mpGameMode = NetRecv::ReadInt16();
	}
}

void NetworkReceiveSound(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		PlaySoundOnNonHiddenActor(NetRecv::ReadInt32());
	}
}

void NetworkReceiveMapNetData(void)
{
    if (Network.IsNetgame() && Network.IsClient())
    {
		int data = NetRecv::ReadInt32();
		int data2 = NetRecv::ReadInt32();
		BP::UserEvent::MapNetReceive(@BP::map, data, data2);
	}
}
